import React, { useState, useEffect } from 'react';
import axios from 'axios';
import { useParams, useNavigate } from 'react-router-dom';
import { Button, IconButton, Box, Typography } from '@mui/material';
import HostnameDialog from './HostnameDialog';
import AddIcon from '@mui/icons-material/Add'; // If you want to use an icon
import { ArrowBack } from '@mui/icons-material';
import DeleteIcon from '@mui/icons-material/Delete';
import Snackbar from '@mui/material/Snackbar';
import Alert from '@mui/material/Alert';
import { DataGrid } from '@mui/x-data-grid';

function HostnamesList() {
    const API_BASE_URL = window._env_.REACT_APP_API_BASE_URL;
    const { category } = useParams();
    const [data, setData] = useState([]);
    const [inputStructure, setInputStructure] = useState({});
    const [outputStructure, setOutputStructure] = useState({});
    const [isDialogOpen, setIsDialogOpen] = useState(false);
    const [selectedHostname, setSelectedHostname] = useState(null);
    const [openSnackbar, setOpenSnackbar] = useState(false);
    const [snackbarMessage, setSnackbarMessage] = useState('');

    const [error, setError] = useState('');
    const navigate = useNavigate();

    const fetchHostnameDetails = async (hostname) => {
        try {
            const token = localStorage.getItem('token');
            const response = await axios.get(`${API_BASE_URL}/${category}/${hostname}`, {
                headers: { Authorization: `Bearer ${token}` }
            });
            return response.data;
        } catch (error) {
            console.error('Error fetching hostname details:', error);
        }
    };

    // Dynamic columns from outputStructure
    const dynamicColumns = Object.keys(outputStructure).map(key => ({
        field: key,
        headerName: key.charAt(0).toUpperCase() + key.slice(1), // Capitalize header name
        flex: 1,
    }));

    // Combine static and dynamic columns
    const columns = [
        { field: 'Hostname', headerName: 'Hostname', width: 150 },
        // Add dynamic columns
        ...dynamicColumns,
        // Delete column
        {
            field: 'delete',
            headerName: 'Delete',
            sortable: false,
            renderCell: (params) => (
                <IconButton onClick={(e) => {
                    e.stopPropagation()
                    handleDelete(params.row.Category, params.row.Hostname)
                }}>
                    <DeleteIcon />
                </IconButton>
            ),
            width: 100,
        },
    ];

    // Format data for Data Grid
    const formattedData = data.map((item, index) => ({
        ...item,
        ...item.Parameters,
        id: item.Id || index  // Ensure a unique ID
    }));

    const handleSnackbarOpen = (message) => {
        setSnackbarMessage(message);
        setOpenSnackbar(true);
    };

    const handleRowClick = async (hostnameId) => {
        const details = await fetchHostnameDetails(hostnameId);
        setSelectedHostname(details); // Make sure this is the data, not the function
        setIsDialogOpen(true);
    };

    const handleAddNew = () => {
        const newHostnameDetails = {
            // Set up initial structure as needed for a new hostname
            Category: category, // Make sure to set the category
            Parameters: {}
        };

        setSelectedHostname(newHostnameDetails);
        setIsDialogOpen(true);
    };

    const fetchHostnames = async () => {
        try {
            const token = localStorage.getItem('token');
            const response = await axios.get(`${API_BASE_URL}/${category}`, {
                headers: {
                    Authorization: `Bearer ${token}`
                }
            });
            setData(response.data); // Assuming setData updates your hostnames list
        } catch (error) {
            console.error('Error fetching hostnames:', error);
            setError('Failed to load hostnames');
        }
    };

    const handleClose = () => {

        fetchHostnames()

        setIsDialogOpen(false);
        setSelectedHostname(null); // Reset the selected hostname state
        // You can also reset other states if necessary
    };

    const handleDelete = async (category, hostname) => {
        try {
            const token = localStorage.getItem('token');
            const response = await axios.delete(`${API_BASE_URL}/${category}/${hostname}`, {
                headers: {
                    Authorization: `Bearer ${token}`
                }
            });

            if (response.status === 200) {
                console.log('Delete successful');
                // Optionally, refresh the list to reflect the deletion
                fetchHostnames(); // Assuming you have a function to refresh the list
            }
        } catch (error) {
            console.error('Error deleting hostname:', error);
            // Handle error (e.g., display an error message)
        }
    };


    useEffect(() => {
        const fetchData = async () => {
            try {
                const token = localStorage.getItem('token');
                // Fetch the output structure
                const structureResponse = await axios.get(`${API_BASE_URL}/rules/${category}`, {
                    headers: {
                        Authorization: `Bearer ${token}`
                    }
                });
                setOutputStructure(JSON.parse(structureResponse.data.output));
                setInputStructure(JSON.parse(structureResponse.data.input));

                // Fetch the category data
                const dataResponse = await axios.get(`${API_BASE_URL}/${category}`, {
                    headers: {
                        Authorization: `Bearer ${token}`
                    }
                });
                setData(dataResponse.data);

            } catch (error) {
                console.error('Error fetching data:', error);
                setError('Failed to load data');
            }
        };

        fetchData();
    }, [category]);

    return (
        <div>
            <Typography variant="h2" gutterBottom>
                Select Host
            </Typography>
            <Button onClick={() => navigate(-1)} startIcon={<ArrowBack />}>
                Back
            </Button>
            <Button onClick={handleAddNew} color="primary" startIcon={<AddIcon />}>
                Add New Hostname
            </Button>

            <Snackbar
                open={openSnackbar}
                autoHideDuration={60000}
                onClose={() => setOpenSnackbar(false)}
            >
                <Alert
                    onClose={() => setOpenSnackbar(false)}
                    severity="success"
                    sx={{ width: '100%' }}
                >
                    {snackbarMessage}
                </Alert>
            </Snackbar>
            {error && <p style={{ color: 'red' }}>{error}</p>}
            <Box sx={{height: '100%'}}>
                <DataGrid
                    rows={formattedData}
                    columns={columns}
                    disableRowSelectionOnClick="true"
                    pageSize={50}
                    onRowClick={(params, e) => {
                        e.stopPropagation()
                        handleRowClick(params.row.Hostname)
                    }
                    }
                    sx={{height: '100%'}}
                /></Box>
       
            <HostnameDialog
                open={isDialogOpen}
                onClose={handleClose}
                hostnameDetails={selectedHostname} // This should be the data, not the function
                inputStructure={inputStructure}
                onSuccessfulSave={handleSnackbarOpen}
                category={category}
            // Other props
            />

        </div>
    );
}

export default HostnamesList;
